/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * SCIDYNE Multi-IO-ARD  10-26-21  Mark Durgin
 * Basic demo for using the eight Analog Outputs
 * Utilizing two MCP4728 4-Channel 12-bit I2C DACs
 * 
 * For compatibility with 3.3V and 5V hardware this program sets the
 * DAC chips VREF to use the internal 2.048 reference and sets a Gain of 1.
 * Therefore, for any DAC Channel Vout = 2.048V * (VALUE / 4095),
 * where VALUE is the 12-bit number written to the input register
 * of a DAC channel.
 * 
 * See MCP4728 datasheet for details
 */

/* * * * * * * * * * * * * * * *
 * Important include files and program constants
 */
#include <Wire.h>       // Needed for Arduino I2C support.
#define WIREPORT Wire  // Set Wire Library: UNO, MEGA use Wire, DUE use Wire1


/* * * * * * * * * * * * * * * * 
 * Setup Runs only once. Execution then 
 * goes to loop
 */
void setup(void) {

   // DAC0 definitions
  #define MCP4728_DAC0_ADDRESS 0x60   // Address of DAC0 on I2C bus. Factory default programmed to be 0x60
  #define AO_0_VALUE   0              // DAC0 Channel A, Analog Output#0 Produces 0.000 Volts
  #define AO_1_VALUE   585            // DAC0 Channel B, Analog Output#1 Produces 0.293 Volts
  #define AO_2_VALUE   1170           // DAC0 Channel C, Analog Output#2 Produces 0.585 Volts
  #define AO_3_VALUE   1755           // DAC0 Channel D, Analog Output#3 Produces 0.878 Volts
  
  // DAC1 definitions
  #define MCP4728_DAC1_ADDRESS 0x61   // Address of DAC1 on I2C bus. Factory default programmed to be 0x61
  #define AO_4_VALUE   2340           // DAC1 Channel A, Analog Output#4 Produces 1.170 Volts
  #define AO_5_VALUE   2925           // DAC1 Channel B, Analog Output#5 Produces 1.463 Volts
  #define AO_6_VALUE   3510           // DAC1 Channel C, Analog Output#6 Produces 1.755 Volts
  #define AO_7_VALUE   4095           // DAC1 Channel D, Analog Output#7 Produces 2.048 Volts
   
  // Local temporary variables created on stack
  uint8_t output_buffer[16];          // Holds DAC bytes to be written via I2C

  // Initialize I2C support
  WIREPORT.begin();       

  // Preload DAC0 values into buffer
  output_buffer[0] = AO_0_VALUE >> 8;      // DAC0 Channel A HB
  output_buffer[1] = AO_0_VALUE & 0xFF;    // DAC0 Channel A LB

  output_buffer[2] = AO_1_VALUE >> 8;      // DAC0 Channel B HB
  output_buffer[3] = AO_1_VALUE & 0xFF;    // DAC0 Channel B LB

  output_buffer[4] = AO_2_VALUE >> 8;      // DAC0 Channel C HB
  output_buffer[5] = AO_2_VALUE & 0xFF;    // DAC0 Channel C LB

  output_buffer[6] = AO_3_VALUE >> 8;      // DAC0 Channel D HB
  output_buffer[7] = AO_3_VALUE & 0xFF;    // DAC0 Channel D LB

  // Preload DAC1 values into buffer
  output_buffer[8] = AO_4_VALUE >> 8;      // DAC1 Channel A HB
  output_buffer[9] = AO_4_VALUE & 0xFF;    // DAC1 Channel A LB

  output_buffer[10] = AO_5_VALUE >> 8;     // DAC1 Channel B HB
  output_buffer[11] = AO_5_VALUE & 0xFF;   // DAC1 Channel B LB

  output_buffer[12] = AO_6_VALUE >> 8;     // DAC1 Channel C HB
  output_buffer[13] = AO_6_VALUE & 0xFF;   // DAC1 Channel C LB

  output_buffer[14] = AO_7_VALUE >> 8;     // DAC1 Channel D HB
  output_buffer[15] = AO_7_VALUE & 0xFF;   // DAC1 Channel D LB
  
   
  /* * * * * * * * * * * * * * * * * * * * * * *
   * Set VREF of each DAC channel. In this example,
   * internal VREF (VREF = 2.048) is selected for all DAC channels
   * See MCP4728 datasheet for details
   */
  // Perform on DAC0 ...
  WIREPORT.beginTransmission(MCP4728_DAC0_ADDRESS);  // Open I2C communication with the MCP4728 DAC0
  WIREPORT.write(0x8F);                              // C2 = 1, C1,C0 = 0,  VrefA - VrefD = 1 (Internal Vref) 
  WIREPORT.endTransmission();                        // Close I2C communications
  // ... and also on DAC1
  WIREPORT.beginTransmission(MCP4728_DAC1_ADDRESS);  // Open I2C communication with the MCP4728 DAC1
  WIREPORT.write(0x8F);                              // C2=1, C1,C0 = 0,  VrefA - VrefD = 1 (Internal Vref) 
  WIREPORT.endTransmission();                        // Close I2C communications

  /* * * * * * * * * * * * * * * * * * * * *
   * Set Gain of each DAC channel. In this example, 
   * Gain of 1 is selected for all DAC channels.
   * See MCP4728 datasheet for details
   */
  // Perform on DAC0 ...
  WIREPORT.beginTransmission(MCP4728_DAC0_ADDRESS);  // Open I2C communication with the MCP4728 DAC0
  WIREPORT.write(0xC0);                              // C2,C1 = 1, C0 = 0,  GainA - GainD = 0 (Gain= 1X) 
  WIREPORT.endTransmission();                        // Close I2C communications
  // ... and also on DAC1
  WIREPORT.beginTransmission(MCP4728_DAC1_ADDRESS);  // Open I2C communication with the MCP4728 DAC1
  WIREPORT.write(0xC0);                              // C2,C1 = 1, C0 = 0,  GainA - GainD = 0 (Gain= 1X) 
  WIREPORT.endTransmission();                        // Close I2C communications


  /* * * * * * * * * *
   * Perform Fast Write to all DAC channels
   */
  // Perform on DAC0 Analog Outputs 0,1,2,3 ...
  WIREPORT.beginTransmission(MCP4728_DAC0_ADDRESS);  // Open I2C communication with the MCP4728 DAC0
  // Analog Output #0
  WIREPORT.write(output_buffer[0]);                  // DAC0 Channel A Value HB, Analog Output#0
  WIREPORT.write(output_buffer[1]);                  // DAC0 Channel A Value LB, Analog Output#0
  // Analog Output #1
  WIREPORT.write(output_buffer[2]);                  // DAC0 Channel B Value HB, Analog Output#1
  WIREPORT.write(output_buffer[3]);                  // DAC0 Channel B Value LB, Analog Output#1
  // Analog Output #2
  WIREPORT.write(output_buffer[4]);                  // DAC0 Channel C Value HB, Analog Output#2
  WIREPORT.write(output_buffer[5]);                  // DAC0 Channel C Value LB, Analog Output#2
  // Analog Output #3
  WIREPORT.write(output_buffer[6]);                  // DAC0 Channel D Value HB, Analog Output#3
  WIREPORT.write(output_buffer[7]);                  // DAC0 Channel D Value LB, Analog Output#3
  WIREPORT.endTransmission();                        // Close I2C communications with the MCP4728 DAC0
  
  // ... and on DAC1 Analog Outputs 4,5,6,7 ...
  WIREPORT.beginTransmission(MCP4728_DAC1_ADDRESS);  // Open I2C communication with the MCP4728 DAC1
  // Analog Output #4
  WIREPORT.write(output_buffer[8]);                  // DAC1 Channel A Value HB, Analog Output#4
  WIREPORT.write(output_buffer[9]);                  // DAC1 Channel A Value LB, Analog Output#4
  // Analog Output #5
  WIREPORT.write(output_buffer[10]);                 // DAC1 Channel B Value HB, Analog Output#5
  WIREPORT.write(output_buffer[11]);                 // DAC1 Channel B Value LB, Analog Output#5
  // Analog Output #6
  WIREPORT.write(output_buffer[12]);                 // DAC1 Channel C Value HB, Analog Output#6
  WIREPORT.write(output_buffer[13]);                 // DAC1 Channel C Value LB, Analog Output#6
  // Analog Output #7
  WIREPORT.write(output_buffer[14]);                 // DAC1 Channel D Value HB, Analog Output#7
  WIREPORT.write(output_buffer[15]);                 // DAC1 Channel D Value LB, Analog Output#7
  WIREPORT.endTransmission();                        // Close I2C communications with the MCP4728 DAC1
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Execution comes here after setup runs only once.
 * Loop continuously does nothing
 */
void loop() { }
